@@ -647,6 +647,19 @@ void CL_FireEntityEvents (frame_t *frame)
 		s1 = &cl_parse_entities[num];
 		if (s1->event)
 			CL_EntityEvent (s1);
+#ifdef QMAX
+		//add stains if moving...
+		if (s1->origin[0]!=s1->old_origin[0]||
+			s1->origin[1]!=s1->old_origin[1]||
+			s1->origin[2]!=s1->old_origin[2])
+		{
+			if (s1->effects & EF_GIB)
+				re.AddStain(s1->origin, 25, 0, -200 ,-200);
+			if (s1->effects & EF_GREENGIB)
+				re.AddStain(s1->origin, 25, -200, 0, -200);
+
+		}
+#endif
 
 		// EF_TELEPORTER acts like an event, but is not cleared each frame
 		if (s1->effects & EF_TELEPORTER)
@@ -1216,11 +1229,11 @@ void CL_AddPacketEntities (frame_t *frame)
 			}
 			else if (effects & EF_GIB)
 			{
-				CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);
+			  CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);
 			}
 			else if (effects & EF_GRENADE)
 			{
-				CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);
+			  CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);
 			}
 			else if (effects & EF_FLIES)
 			{
@@ -1299,7 +1312,7 @@ void CL_AddPacketEntities (frame_t *frame)
 			// RAFAEL
 			else if (effects & EF_GREENGIB)
 			{
-				CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);				
+			  //CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);				
 			}
 			// RAFAEL
 			else if (effects & EF_IONRIPPER)
