@@ -1160,37 +1160,36 @@ void pStunRotateThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alpha, f
 
 void CL_StunBlast (vec3_t pos, vec3_t color, float size)
 {
-	vec3_t dir, angles;
 	int i;
 
 	for (i=0;i<8;i++)
-	{
-		setupParticle (
-			0,		0,		0,
-			pos[0],		pos[1],		pos[2],
-			crandom()*size,		crandom()*size,		crandom()*size,
-			0,		0,		0,
-			color[0],		color[1],		color[2],
-			0,	0,	0,
-			.5,		-1 / (0.8+frand()*0.2),
-			size,	0,			
-			particle_generic,
-			0,
-			0, false);
-	}
-
+	  {
+	    setupParticle (
+			   0,		0,		0,
+			   pos[0],		pos[1],		pos[2],
+			   crandom()*size,		crandom()*size,		crandom()*size,
+			   0,		0,		0,
+			   color[0],		color[1],		color[2],
+			   0,	0,	0,
+			   .5,		-1 / (0.8+frand()*0.2),
+			   size,	0,			
+			   particle_generic,
+			   0,
+			   0, false);
+	  }
+	
 	setupParticle (
-		crandom()*360,		crandom()*360,		crandom()*360,
-		pos[0],		pos[1],		pos[2],
-		75,		300,	75,
-		0,		0,		0,
-		color[0],		color[1],		color[2],
-		0,	0,	0,
-		.75,		-1 / (0.8+frand()*0.2),
-		size,	size*3,			
-		particle_smoke,
-		PART_ANGLED,
-		pStunRotateThink, true);
+		       crandom()*360,		crandom()*360,		crandom()*360,
+		       pos[0],		pos[1],		pos[2],
+		       75,		300,	75,
+		       0,		0,		0,
+		       color[0],		color[1],		color[2],
+		       0,	0,	0,
+		       .75,		-1 / (0.8+frand()*0.2),
+		       size,	size*3,			
+		       particle_smoke,
+		       PART_ANGLED,
+		       pStunRotateThink, true);
 }
 
 void CL_LaserStun (vec3_t pos, vec3_t direction, vec3_t color, float size)
@@ -1421,7 +1420,7 @@ void CL_FlameTrail (vec3_t start, vec3_t end, float size)
 }
 
 void CL_Flame (vec3_t start)
-{	int i;
+{      
 	cparticle_t *p;
 
 
@@ -1648,7 +1647,7 @@ void pExplosionThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alpha, fl
 #define EXPLODESTAININTESITY 75
 void CL_Explosion_Particle (vec3_t org, float size, qboolean large, qboolean rocket)
 {
-	cparticle_t *p;
+    cparticle_t *p;
 
 	if (large)
 	{	
@@ -1709,7 +1708,8 @@ void CL_Explosion_Particle (vec3_t org, float size, qboolean large, qboolean roc
 					particle_inferno,
 					0,
 					NULL,0);
-	}*/
+	}
+  */
 }
 
 
@@ -1733,68 +1733,67 @@ void pDisruptExplosionThink (cparticle_t *p, vec3_t org, vec3_t angle, float *al
 
 void CL_Disruptor_Explosion_Particle (vec3_t org, float size)
 {
-	int i;
-	float	alphastart = 1,
-			alphadecel = -5;
-	cparticle_t *p;
-
-	re.AddStain(org, size, -EXPLODESTAININTESITY,-EXPLODESTAININTESITY,-EXPLODESTAININTESITY);
-
-	//now add main sprite
-	{
-		p = setupParticle (
-					0,		0,		0,
-					org[0],	org[1],	org[2],
-					0,		0,		0,
-					0,		0,		0,
-					255,	255,	255,
-					0,		0,		0,
-					alphastart,		alphadecel,
-					size,	0,			
-					particle_dexplosion1,
-					PART_DEPTHHACK_SHORT,
-					pDisruptExplosionThink, true);
-
-		if (p)
-		{
-			float lightsize = size/150.0;
-
-			addParticleLight (p,
-						size*1.0f, 0,
-						1, 1, 1);
-			addParticleLight (p,
-						size*1.25f, 0,
-						0.75, 0, 1);
-			addParticleLight (p,
-						size*1.65f, 0,
-						0.25, 0, 1);
-			addParticleLight (p,
-						size*1.9f, 0,
-						0, 0, 1);
-		}
-	}
-/*
-	alphastart /= 10.0;
-	alphadecel /= 10.0;
-
-	//now make volumizers...
-	for (i=0;i<256;i++)
-	{
-		int	size2 = size*2;	
-
-		setupParticle (
-					0,		0,		0,
-					org[0]+ ((rand()%size2)-size)*0.1,	org[1]+ ((rand()%size2)-size)*0.1,	org[2]+ ((rand()%size2)-size)*0.1,
-					((rand()%size2)-size)*2.0,	((rand()%size2)-size)*2.0,	((rand()%size2)-size)*2.0,
-					0,		0,		0,
-					100,	100,	200,
-					0,		0,		0,
-					alphastart,		alphadecel,
-					size/3.0,	-10,			
-					particle_smoke,
-					PART_DIRECTION,
-					pSplashThink,true);
-	}*/
+  float	alphastart = 1, 
+    alphadecel = -5;
+  cparticle_t *p;
+  
+  re.AddStain(org, size, -EXPLODESTAININTESITY,-EXPLODESTAININTESITY,-EXPLODESTAININTESITY);
+  
+  //now add main sprite
+  {
+    p = setupParticle (
+		       0,		0,		0,
+		       org[0],	org[1],	org[2],
+		       0,		0,		0,
+		       0,		0,		0,
+		       255,	255,	255,
+		       0,		0,		0,
+		       alphastart,		alphadecel,
+		       size,	0,			
+		       particle_dexplosion1,
+		       PART_DEPTHHACK_SHORT,
+		       pDisruptExplosionThink, true);
+    
+    if (p)
+      {
+	//float lightsize = size/150.0;
+	
+	addParticleLight (p,
+			  size*1.0f, 0,
+			  1, 1, 1);
+	addParticleLight (p,
+			  size*1.25f, 0,
+			  0.75, 0, 1);
+	addParticleLight (p,
+			  size*1.65f, 0,
+			  0.25, 0, 1);
+	addParticleLight (p,
+			  size*1.9f, 0,
+			  0, 0, 1);
+      }
+  }
+  /*
+    alphastart /= 10.0;
+    alphadecel /= 10.0;
+    
+    //now make volumizers...
+    for (i=0;i<256;i++)
+    {
+    int	size2 = size*2;	
+    
+    setupParticle (
+    0,		0,		0,
+    org[0]+ ((rand()%size2)-size)*0.1,	org[1]+ ((rand()%size2)-size)*0.1,	org[2]+ ((rand()%size2)-size)*0.1,
+    ((rand()%size2)-size)*2.0,	((rand()%size2)-size)*2.0,	((rand()%size2)-size)*2.0,
+    0,		0,		0,
+    100,	100,	200,
+    0,		0,		0,
+    alphastart,		alphadecel,
+    size/3.0,	-10,			
+    particle_smoke,
+    PART_DIRECTION,
+    pSplashThink,true);
+    }*/
 }
 
 /*
@@ -1815,11 +1814,11 @@ void pRainSplashThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alpha, f
 
 void pWeatherFXThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alpha, float *size, int *image, float *time)
 {
-	float length;
-	int i;
-	vec3_t len, ang;
-
-	VectorSubtract(p->angle, org, len);
+  float length;
+  int i;
+  vec3_t len;
+  
+  VectorSubtract(p->angle, org, len);
 	{
 		float time1, time2;
 
@@ -2194,8 +2193,7 @@ CL_ParticleEffectSparks
 
 void pSparksThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alpha, float *size, int *image, float *time)
 {
-	vec3_t dir;
-	int i;
+  int i;
 
 	//setting up angle for sparks
 	{
@@ -2217,23 +2215,23 @@ void CL_ParticleEffectSparks (vec3_t org, vec3_t dir, vec3_t color, int count)
 	int			i;
 	float		d;
 	for (i=0 ; i<count ; i++)
-	{
-		d = rand()&7;
-		setupParticle (
-			0,	0,	0,
-			org[0]+((rand()&3)-2),	org[1]+((rand()&3)-2),	org[2]+((rand()&3)-2),
-			//crandom()*20 + dir[0]*75,crand()*20 + dir[1]*75,
-			//crand()*20 + dir[2]*75,
-			20+dir[0]*75,20+dir[1]*75,20+dir[2]*75,
-			0,		0,		0,
-			color[0],		color[1],		color[2],
-			0,	0,	0,
-			0.75,		-1.5 / (0.5 + frand()*0.3),
-			0.5,			0,			
-			particle_generic,
-			PART_GRAVITY|PART_DIRECTION,
-			pSparksThink,true);
-	}
+	  {
+	    d = rand()&7;
+	    setupParticle (
+			   0,	0,	0,
+			   org[0]+((rand()&3)-2),	org[1]+((rand()&3)-2),	org[2]+((rand()&3)-2),
+			   //crandom()*20 + dir[0]*75,crand()*20 + dir[1]*75,
+			   //crand()*20 + dir[2]*75,
+			   20+dir[0]*75,20+dir[1]*75,20+dir[2]*75,
+			   0,		0,		0,
+			   color[0],		color[1],		color[2],
+			   0,	0,	0,
+			   0.75,		-1.5 / (0.5 + frand()*0.3),
+			   0.5,			0,			
+			   particle_generic,
+			   PART_GRAVITY|PART_DIRECTION,
+			   pSparksThink,true);
+	  }
 }
 
 /*
@@ -2418,11 +2416,14 @@ void pExplosionSparksThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alp
 	p->thinknext = true;
 }
 
+// This is buggy, not sure why... -bb 10/08/02
+/*
 void CL_ExplosionParticles (vec3_t org, float scale)
 {
 	vec3_t vel;
 	int			i;
 
+  printf("%d %d %d %g\n", org[0], org[1], org[2], scale);
 	for (i=0 ; i<256 ; i++)
 	{
 		VectorSet(vel, crandom(), crandom(), crandom());
@@ -2443,7 +2444,38 @@ void CL_ExplosionParticles (vec3_t org, float scale)
 			pExplosionSparksThink, true);
 	}
 }
+*/
+void CL_ExplosionParticles (vec3_t org)
+{
+	int			i, j;
+	cparticle_t	*p;
 
+	for (i=0 ; i<256 ; i++)
+	{
+		if (!free_particles)
+			return;
+		p = free_particles;
+		free_particles = p->next;
+		p->next = active_particles;
+		active_particles = p;
+
+		p->time = cl.time;
+		p->color[0] = 255;
+		p->color[1] = 0;
+		p->color[2] = rand() & 7;
+		for (j=0 ; j<3 ; j++)
+		{
+			p->org[j] = org[j] + ((rand()%32)-16);
+			p->vel[j] = (rand()%384)-192;
+		}
+
+		p->accel[0] = p->accel[1] = 0;
+		p->accel[2] = -PARTICLE_GRAVITY;
+		p->alpha = 1.0;
+
+		p->alphavel = -0.8 / (0.5 + frand()*0.3);
+	}
+}
 /*
 ===============
 CL_BigTeleportParticles
@@ -2489,7 +2521,6 @@ Wall impact puffs
 #define pBlasterMaxSize 5
 void pBlasterThink (cparticle_t *p, vec3_t org, vec3_t angle, float *alpha, float *size, int *image, float *time)
 {
-	int i;
 	vec3_t len;
 	VectorSubtract(p->angle, org, len);
 	
@@ -2815,7 +2846,7 @@ void CL_RocketTrail (vec3_t start, vec3_t end, centity_t *old)
 
 		if (!free_particles)
 			return;
-/*
+
 		//flame from rocket
 		setupParticle (
 			0,	0,	0,
@@ -2829,8 +2860,7 @@ void CL_RocketTrail (vec3_t start, vec3_t end, centity_t *old)
 			particle_generic,
 			0,
 			NULL,0);
-//*/
-//*
+
 		//falling particles
 		if ( (rand()&7) == 0)
 		{
@@ -2847,7 +2877,6 @@ void CL_RocketTrail (vec3_t start, vec3_t end, centity_t *old)
 				PART_GRAVITY,
 				NULL,0);
 		}
-//*/
 		VectorAdd (move, vec, move);
 	}
 }
@@ -3006,7 +3035,6 @@ void CL_RailTrail (vec3_t start, vec3_t end)
 	vec3_t		vec, point;
 	int			i;
 	float		len;
-	float		dec;
 	vec3_t		right, up;
 	qboolean	colored = (cl_railtype->value!=1);
 
@@ -3357,10 +3385,10 @@ void CL_TrapParticles (entity_t *ent)
 {
 	int colors[][3] =
 	{
-		255, 200, 150,
-		255, 200, 100,
-		255, 200, 50,
-		0
+		{255, 200, 150},
+		{255, 200, 100},
+		{255, 200, 50},
+		{0, 0, 0}
 	};
 	vec3_t		move;
 	vec3_t		vec;
@@ -3425,7 +3453,7 @@ void CL_TrapParticles (entity_t *ent)
 					dir[2] = k * 8;
 		
 					VectorNormalize (dir);						
-					vel = 50 + rand()&63;
+					vel = 50 + (rand()&63);
 
 					index = rand()&3;
 
