@@ -48,7 +48,21 @@ cvar_t	*cl_predict;
 cvar_t	*cl_maxfps;
 cvar_t	*cl_drawfps;
 cvar_t	*cl_gun;
+#ifdef QMAX
+cvar_t	*cl_blood;
 
+cvar_t	*cl_railred;
+cvar_t	*cl_railgreen;
+cvar_t	*cl_railblue;
+cvar_t	*cl_railtype;
+
+cvar_t	*cl_3dcam;
+cvar_t	*cl_3dcam_angle;
+cvar_t	*cl_3dcam_chase;
+cvar_t	*cl_3dcam_dist;
+cvar_t	*cl_3dcam_alpha;
+cvar_t	*cl_3dcam_adjust;
+#endif
 cvar_t	*cl_add_particles;
 cvar_t	*cl_add_lights;
 cvar_t	*cl_add_entities;
@@ -1448,7 +1462,25 @@ void CL_InitLocal (void)
 //	cl_minfps = Cvar_Get ("cl_minfps", "5", 0);
 	cl_maxfps = Cvar_Get ("cl_maxfps", "90", 0);
 	cl_drawfps = Cvar_Get("cl_drawfps","0",CVAR_ARCHIVE); // FPS hack
+
+#ifdef QMAX
+	cl_blood = Cvar_Get ("cl_blood", "1", 0);
 	
+	//psychospaz -- railgun fun
+	cl_railred = Cvar_Get ("cl_railred", "20", CVAR_ARCHIVE);
+	cl_railgreen = Cvar_Get ("cl_railgreen", "50", CVAR_ARCHIVE);
+	cl_railblue = Cvar_Get ("cl_railblue", "175", CVAR_ARCHIVE);
+	cl_railtype = Cvar_Get ("cl_railtype", "1", CVAR_ARCHIVE);
+	//3d camera
+	cl_3dcam = Cvar_Get ("cl_3dcam", "0", CVAR_ARCHIVE);
+	cl_3dcam_angle = Cvar_Get ("cl_3dcam_angle", "0", CVAR_ARCHIVE);
+	cl_3dcam_dist = Cvar_Get ("cl_3dcam_dist", "50", CVAR_ARCHIVE);
+	cl_3dcam_alpha = Cvar_Get ("cl_3dcam_alpha", "1", CVAR_ARCHIVE);
+	cl_3dcam_chase = Cvar_Get ("cl_3dcam_chase", "1", CVAR_ARCHIVE);
+	cl_3dcam_adjust = Cvar_Get ("cl_3dcam_adjust", "1", CVAR_ARCHIVE);
+	//cl_playermodel_default = Cvar_Get ("cl_playermodel_default", DEFAULTSKIN, CVAR_USERINFO);
+#endif
+
 	cl_upspeed = Cvar_Get ("cl_upspeed", "200", 0);
 	cl_forwardspeed = Cvar_Get ("cl_forwardspeed", "200", 0);
 	cl_sidespeed = Cvar_Get ("cl_sidespeed", "200", 0);
