@@ -21,6 +21,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 //define	PARANOID			// speed sapping error checking
 
+#ifdef __x86_64__
+#define INT long int
+#else
+#define INT int
+#endif
+
 #include <math.h>
 #include <string.h>
 #include <stdarg.h>
