@@ -217,6 +217,8 @@ void *Sys_GetGameAPI (void *parms)
 	char	*str_p;
 #if defined __i386__
 	const char *gamename = "gamei386.so";
+#elif defined __x86_64__
+	const char *gamename = "gamex86_64.so";
 #elif defined __alpha__
 	const char *gamename = "gameaxp.so";
 #elif defined __powerpc__
