@@ -188,7 +188,7 @@ static void ApplyChanges( void *unused )
 	{
 		Cvar_Set( "vid_ref", "glx" );
 		// below is wrong if we use different libs for different GL reflibs
-		Cvar_Get( "gl_driver", "libGL.so", CVAR_ARCHIVE ); // ??? create if it doesn't exit
+		Cvar_Get( "gl_driver", "libGL.so.1", CVAR_ARCHIVE ); // ??? create if it doesn't exit
 		if (gl_driver->modified)
 			vid_ref->modified = true;
 	}
@@ -196,7 +196,7 @@ static void ApplyChanges( void *unused )
 	{
 		Cvar_Set( "vid_ref", "sdlgl" );
 		// below is wrong if we use different libs for different GL reflibs
-		Cvar_Get( "gl_driver", "libGL.so", CVAR_ARCHIVE ); // ??? create if it doesn't exist
+		Cvar_Get( "gl_driver", "libGL.so.1", CVAR_ARCHIVE ); // ??? create if it doesn't exist
 		if (gl_driver->modified)
 			vid_ref->modified = true;
 	}
@@ -250,6 +250,16 @@ void VID_MenuInit( void )
 		"[1152 864 ]",
 		"[1280 1024]",
 		"[1600 1200]",
+		"[2048 1536]",
+		"[1024 480 ]", /* sony vaio pocketbook */
+		"[1152 768 ]", /* Apple TiBook */
+		"[1280 854 ]", /* apple TiBook */
+		"[640 400  ]", /* generic 16:10 widescreen resolutions */
+ 		"[800 500  ]", /* as found on many modern notebooks    */
+ 		"[1024 640 ]",
+ 		"[1280 800 ]",
+ 		"[1680 1050]",
+ 		"[1920 1200]",
 		0
 	};
 	static const char *yesno_names[] =
