@@ -788,7 +788,10 @@ void R_FloodFillSkin( byte *skin, int skinwidth, int skinheight )
 		filledcolor = 0;
 		// attempt to find opaque black
 		for (i = 0; i < 256; ++i)
-			if (d_8to24table[i] == (255 << 0)) // alpha 1.0
+			/* if (d_8to24table[i] == (255 << 0)) // alpha 1.0
+			 * ENDIAN problem, fix by xvi
+			 */
+			if (LittleLong(d_8to24table[i]) == (255 << 0)) // alpha 1.0
 			{
 				filledcolor = i;
 				break;
@@ -1253,6 +1256,11 @@ image_t *GL_LoadPic (char *name, byte *pic, int width, int height, imagetype_t t
 {
 	image_t		*image;
 	int			i;
+#ifdef RETEX
+  miptex_t 	*mt;
+  int 		len;
+  char 		s[128];
+#endif
 
 	// find a free image_t
 	for (i=0, image=gltextures ; i<numgltextures ; i++,image++)
@@ -1277,6 +1285,23 @@ image_t *GL_LoadPic (char *name, byte *pic, int width, int height, imagetype_t t
 	image->height = height;
 	image->type = type;
 
+#ifdef RETEX
+  len = strlen(name);
+  strcpy(s,name);
+  
+  if (!strcmp(s+len-4, ".tga") || !strcmp(s+len-4, ".jpg") || !strcmp(s+len-4, ".png"))
+    {
+      s[len-3] = 'w';	s[len-2] = 'a';	s[len-1] = 'l';
+      ri.FS_LoadFile (s, (void **)&mt);	//load .wal file
+      
+      if (mt) {
+	image->width = LittleLong (mt->width);
+	image->height = LittleLong (mt->height);
+	ri.FS_FreeFile ((void *)mt);
+      }
+    }
+#endif
+
 	if (type == it_skin && bits == 8)
 		R_FloodFillSkin(pic, width, height);
 
@@ -1371,13 +1396,21 @@ image_t	*GL_FindImage (char *name, imagetype_t type)
 	int		i, len;
 	byte	*pic, *palette;
 	int		width, height;
-
+	char *ptr;
+	
 	if (!name)
 		return NULL;	//	ri.Sys_Error (ERR_DROP, "GL_FindImage: NULL name");
 	len = strlen(name);
 	if (len<5)
 		return NULL;	//	ri.Sys_Error (ERR_DROP, "GL_FindImage: bad name: %s", name);
 
+#ifndef _WIN32
+	// fix backslashes
+	while ((ptr=strchr(name,'\\'))) {
+	  *ptr = '/';
+	}
+#endif
+
 	// look for it
 	for (i=0, image=gltextures ; i<numgltextures ; i++,image++)
 	{
