@@ -77,6 +77,7 @@ static int keyq_tail=0;
 static int window_width;
 static int window_height;
 
+extern char display_name[];
 
 /*****************************************************************************/
 /* MOUSE                                                                     */
@@ -100,6 +101,9 @@ static cvar_t	*in_mouse;
 static qboolean	mlooking;
 static qboolean mouse_active = false;
 
+// stencilbuffer shadows   
+qboolean have_stencil = false;
+
 // state struct passed in Init
 static in_state_t	*in_state;
 
@@ -725,6 +729,16 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
 {
   int width, height;
   int attrib[] = {
+    GLX_RGBA,
+    GLX_RED_SIZE, 1,
+    GLX_GREEN_SIZE, 1,
+    GLX_BLUE_SIZE, 1,
+    GLX_DOUBLEBUFFER,
+    GLX_DEPTH_SIZE, 1,
+    GLX_STENCIL_SIZE, 1,
+    None
+  };
+  int attrib_nostencil[] = {
     GLX_RGBA,
     GLX_RED_SIZE, 1,
     GLX_GREEN_SIZE, 1,
@@ -761,10 +775,10 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
   window_width = width;
   window_height = height;
 
-  // destroy the existing window
+  // destroy the existing window (Commented out on Mon Sep 20 15:09:02 MEST 2004)
   GLimp_Shutdown ();
 
-  if (!(dpy = XOpenDisplay(NULL))) {
+  if (!(dpy = XOpenDisplay(display_name))) {
     fprintf(stderr, "Error couldn't open the X display\n");
     return rserr_invalid_mode;
   }
@@ -777,10 +791,47 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
 
   visinfo = qglXChooseVisual(dpy, scrnum, attrib);
   if (!visinfo) {
-    fprintf(stderr, "Error couldn't get an RGB, Double-buffered, Depth visual\n");
-    return rserr_invalid_mode;
+    fprintf(stderr, "Error couldn't get an RGBA, DOUBLEBUFFER, DEPTH, STENCIL visual\n");
+    visinfo = qglXChooseVisual(dpy, scrnum, attrib_nostencil);
+    if (!visinfo) {
+      fprintf(stderr, "E: couldn't get an RGBA, DOUBLEBUFFER, DEPTH visual\n");
+      return rserr_invalid_mode;
+    }
   }
 
+	/* do some pantsness */
+	if ( qglXGetConfig )
+	{
+		int red_bits, blue_bits, green_bits, depth_bits, alpha_bits;
+
+		qglXGetConfig(dpy, visinfo, GLX_RED_SIZE, &red_bits);
+		qglXGetConfig(dpy, visinfo, GLX_BLUE_SIZE, &blue_bits);
+		qglXGetConfig(dpy, visinfo, GLX_GREEN_SIZE, &green_bits);
+		qglXGetConfig(dpy, visinfo, GLX_DEPTH_SIZE, &depth_bits);
+		qglXGetConfig(dpy, visinfo, GLX_ALPHA_SIZE, &alpha_bits);
+
+		ri.Con_Printf(PRINT_ALL, "I: got %d bits of red\n", red_bits);
+		ri.Con_Printf(PRINT_ALL, "I: got %d bits of blue\n", blue_bits);
+		ri.Con_Printf(PRINT_ALL, "I: got %d bits of green\n", green_bits);
+		ri.Con_Printf(PRINT_ALL, "I: got %d bits of depth\n", depth_bits);
+		ri.Con_Printf(PRINT_ALL, "I: got %d bits of alpha\n", alpha_bits);
+	}
+
+	/* stencilbuffer shadows */
+	if ( qglXGetConfig )
+	{
+		int stencil_bits;
+
+		if (!qglXGetConfig(dpy, visinfo, GLX_STENCIL_SIZE, &stencil_bits)) {
+			ri.Con_Printf(PRINT_ALL, "I: got %d bits of stencil\n", stencil_bits);
+			if (stencil_bits >= 1) {
+				have_stencil = true;
+			}
+		}
+	}
+
+
+
   /* window attributes */
   attr.background_pixel = 0;
   attr.border_pixel = 0;
@@ -828,17 +879,31 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
 */
 void GLimp_Shutdown( void )
 {
-  if (dpy) {
-    if (ctx)
-      qglXDestroyContext(dpy, ctx);
-    if (win)
-      XDestroyWindow(dpy, win);
-    XCloseDisplay(dpy);
-  }
-  ctx = NULL;
-  dpy = NULL;
-  win = 0;
-  ctx = NULL;
+	uninstall_grabs();
+	mouse_active = false;
+#ifdef __linux__
+	dgamouse = false;
+#endif
+
+	if (dpy) {
+		if (ctx)
+			qglXDestroyContext(dpy, ctx);
+		if (win)
+			XDestroyWindow(dpy, win);
+		XUngrabKeyboard(dpy, CurrentTime);
+		XCloseDisplay(dpy);
+	}
+	ctx = NULL;
+	dpy = NULL;
+	win = 0;
+/*	
+	qglXChooseVisual             = NULL;
+	qglXCreateContext            = NULL;
+	qglXDestroyContext           = NULL;
+	qglXMakeCurrent              = NULL;
+	qglXCopyContext              = NULL;
+	qglXSwapBuffers              = NULL;
+*/	
 }
 
 /*
@@ -860,8 +925,9 @@ int GLimp_Init( void *hinstance, void *wndproc )
 		qglXMakeCurrent              =  GPA("glXMakeCurrent");
 		qglXCopyContext              =  GPA("glXCopyContext");
 		qglXSwapBuffers              =  GPA("glXSwapBuffers");
+		qglXGetConfig                =  GPA("glXGetConfig");
 		
-  return true;
+		return true;
 	}
 	
 	return false;
