@@ -35,7 +35,7 @@
 #include "../client/keys.h"
 #include "../solaris/rw_solaris.h"
 
-#include "q2.xbm"
+#include "q2icon.xbm"
 
 
 /*****************************************************************************/
@@ -250,6 +250,9 @@ static cvar_t	*vid_ypos;			// Y coordinate of window position
 
 static qboolean	mlooking;
 
+// stencilbuffer shadows   
+qboolean have_stencil = false;
+
 // state struct passed in Init
 static in_state_t	*in_state;
 
@@ -988,7 +991,7 @@ static qboolean SWimp_InitGraphics( qboolean fullscreen )
     Pixmap icon;
     XWMHints hints;
     icon = XCreateBitmapFromData( dpy, DefaultRootWindow( dpy ),
-				  (char *)q2_bits, q2_width, q2_height );
+				  (char *)q2icon_bits, q2icon_width, q2icon_height );
 				  /*				  BlackPixel( dpy, DefaultScreen( dpy ) ),
 				  WhitePixel( dpy, DefaultScreen( dpy ) ),
 				  1 ); */
